const fs = require('fs');
const axios = require('axios').default;
const zipdir = require('zip-dir');
const callService = require('../../tools/callService');
const callICService = (serviceKey, serviceParams) => {
  return callService({
    projectId: '702926627029266270292662',
    serviceKey: serviceKey,
    params: serviceParams || {}
  })
}

class ProjectManager {

  /**
   * 
   * @param {Object} params
   * @param {Object} [params.accessToken] - Personal Access Token
   * @param {Object} [params.projectAccessToken] - Project Access Token
   * @param {Object} [params.projectId] - Project id
   */
  constructor (params) {
    this._accessToken = params.accessToken || process.env.IC_ACCESS_TOKEN;
    this._projectAccessToken = params.projectAccessToken || process.env.IC_PROJECT_ACCESS_TOKEN;
    this._projectId = params.projectId;
  }

  /**
   * @returns {Promise<{globalVars:{},models:[Object],functions:[Object],services:[Object],daemons:[Object],webapps:[Object],mobileapps:[Object]}>}
   */
  async getBetaSetup () {
    let output = await callICService('getProjectBetaSetup', {
      accessToken: this._accessToken,
      projectId: this._projectId,
    })
    return output;
  }

  /**
   * @returns {Promise}
   * @param {Object} setup
   * @param {Object} setup.globalVars
   * @param {Array} setup.models
   * @param {Array} setup.functions
   * @param {Array} setup.services
   * @param {Array} setup.daemons
   * @param {Array} setup.webapps
   * @param {Array} setup.mobileapps
   */
  async updateBetaSetup (setup) {
    let output = await callICService('updateProjectBetaSetup', {
      accessToken: this._accessToken,
      projectId: this._projectId,
      setup: setup
    })
    return output;
  }

  /**
   * @returns {Promise}
   * @param {string} customWebappId
   * @param {Object} data
   * @param {string} [data.description]
   * @param {string} [data.folderPath]
   * @param {string} [data.zipPath]
   * @param {Buffer} [data.zipBuffer]
   */
  async uploadCustomWebappVersion (customWebappId, data) {
    let zipBuffer = null;
    if (data.folderPath) {
      zipBuffer = await zipdir(data.folderPath);
    } else if (data.zipPath) {
      zipBuffer = fs.readFileSync(data.zipPath)
    } else if (data.zipBuffer) {
      zipBuffer = data.zipBuffer;
    } else {
      throw 'Invalid source, please pass folder or zip path';
    }
    let uploadReqOut = await callICService('requestCustomWebappUploadUrl', {
      accessToken: this._accessToken,
      projectAccessToken: this._projectAccessToken,
      projectId: this._projectId,
      customWebappId: customWebappId,
      versionDescription: data.description || '',
    })
    await axios({
      url: uploadReqOut.url,
      method: 'PUT',
      headers: {
        'Content-Type': 'application/zip'
      },
      data: zipBuffer,
      maxContentLength: Infinity,
      maxBodyLength: Infinity
    })
    return;
  }

}

module.exports = ProjectManager;